:li

!!! Listing 1

/****************************************************************************
 *
 * Datei.h
 *  
 * Quelle: Seminarunterlagen der BREDEX GmbH
 *
 ****************************************************************************/

#ifndef _Datei_hh
#define _Datei_hh

#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

class Datei {
  private:
    FILE* fp;      // Datei-Zeiger (NULL: nichts geoeffnet)
    int   size;    // Groesse (-1: noch nicht ermittelt)

  public:
    Datei (const char* name);   // Konstruktor
    ~Datei ();                  // Destruktor

    /* Groesse (berechnen und) zurueckliefern
     *  - nur fuer Variablen aufrufbar
     */
    int groesse ();

    /* Datei auf Stream s ausgeben
     *  - auch fuer Konstanten aufrufbar
     */
    void printOn (ostream& s) const;

  // Kopieren und Zuweisen ist verboten
  private:
    Datei (const Datei&) {}                   // Copy-Konstruktor
    const Datei& operator= (const Datei&) {}  // Zuweisung
};

// Standard-Ausgabeoperator
inline ostream& operator<< (ostream& strm, const Datei& s)
{
    s.printOn (strm);    // Datei auf Stream s ausgeben
    return strm;         // Stream zur Verkettung zurueckliefern
}

#endif  /* _Datei_hh */

!!! Listing 2

/****************************************************************************
 *
 * Datei.cc / Datei.cpp
 *  
 * Quelle: Seminarunterlagen der BREDEX GmbH
 *
 ****************************************************************************/

// Header-Dateien
#include "Datei.h"
#include <sys/stat.h>
#include <unistd.h>

/* fuer Linux; eventuell 
#include <sys/types.h>
statt unistd.h		*/

// Konstruktor
Datei::Datei (const char* name) {
    fp = fopen (name, "r");     // Datei oeffnen
    size = -1;
}

// Destruktor
Datei::~Datei () {
    if (fp != NULL) {
        fclose (fp);            // Datei schliessen
    }
}

// Groesse (berechnen und) zurueckliefern
int Datei::groesse () {
    if (fp == NULL) {
            return 0;
    }
    if (size == -1) {           // ggf. Groesse berechnen
        struct stat buf;
        if (fstat(fileno(fp),&buf) == 0) {
            size = buf.st_size;
        }
    }
    return size;
}

// Datei auf Stream s ausgeben
void Datei::printOn (ostream& s) const {
    if (fp != NULL) {
        int c;
        while ((c = fgetc(fp)) != EOF) {
            s.put((char) c);
        }
    }
}


!!! Listing 3

/****************************************************************************
 *
 * Anwendungsbeispiel zur Klasse Datei
 *
 * Quelle: Seminarunterlagen der BREDEX GmbH
 *
 ****************************************************************************/

#include "Datei.h"
#include <iostream.h>

main (int argc, char** argv)
{
    // fuer jeden Parameter aus der Kommandozeile
    for (int i=1; i<argc; i++) {
        Datei d(argv[i]);       // koennte eigentlich Konstante sein

        /* Datei-Groesse ausgeben
         */
        cout << "\n**** Datei:   " << argv[i] << endl;
        cout << "**** Groesse: " << d.groesse() << endl;;

        /* und, falls gewuenscht, Inhalt ausgeben
         */
        cout << "**** Inhalt ausgeben (j/n) ?  [n] : ";
        if (cin.get() == 'j') {
            cout << d << endl;
        }
    }
}


!!! Listing 4

/****************************************************************************
 *
 * Datei.h mit "mutable"
 *  
 * Quelle: Seminarunterlagen der BREDEX GmbH
 *
 ****************************************************************************/

#ifndef _Datei_hh
#define _Datei_hh

#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

class Datei {
  private:
    FILE*       fp;      // Datei-Zeiger (NULL: nichts geoeffnet)
    mutable int size;    // Groesse (NEU: auch fuer Konstanten aenderbar)

  public:
    Datei (const char* name);   // Konstruktor
    ~Datei ();                  // Destruktor

    /* Groesse (berechnen und) zurueckliefern
     *  - NEU: auch fuer Konstanten aufrufbar
     */
    int groesse () const;

    /* Datei auf Stream s ausgeben
     *  - auch fuer Konstanten aufrufbar
     */
    void printOn (ostream& s) const;

  // Kopieren und Zuweisen ist verboten
  private:
    Datei (const Datei&) {}                   // Copy-Konstruktor
    const Datei& operator= (const Datei&) {}  // Zuweisung
};

// Standard-Ausgabeoperator
inline ostream& operator<< (ostream& strm, const Datei& s)
{
    s.printOn (strm);    // Datei auf Stream s ausgeben
    return strm;         // Stream zur Verkettung zurueckliefern
}

#endif  /* _Datei_hh */


!!! Listing 5

/****************************************************************************
 *
 * Elementfunktion groesse() mit entfernter Konstantheit
 *  
 * Quelle: Seminarunterlagen der BREDEX GmbH
 *
 ****************************************************************************/

/* Groesse (berechnen und) zurueckliefern
 *  - NEU: Konstanten--Elementfunktion, die Komponenten aendert
 */
int Datei::groesse () const {
    if (fp == NULL) {
            return 0;
    }
    if (size == -1) {           // ggf. Groesse berechnen
        struct stat buf;
        if (fstat(fileno(fp),&buf) == 0) {
            // NEU: Konstantheit von this wegcasten
            ((Datei*)this)->size = buf.st_size;
        }
    }
    return size;
}


